/*
 *  ILBM->ASCII.C
 *
 *  Taken from IFF2ASCII.AMOS (c)1994 Matthew Stratfold M.P.Stratfold@open.ac.uk
 *  Improved by Tobias Ferber on Thu Mar 24 04:39:58 1994, ukjg@rz.uni-karlsruhe.de
 *
 *  The Conversion Table is inspired by Jorn Barger's alt.ascii-art FAQ
 *  which had the table by Rob Harley(robert@vlsi.cs.caltech.edu) in it.
 *
 *  This program is BeerWare - if you like it go and buy yourself a beer :-)
 */

#include "iffp/ilbmapp.h"
#include <stdio.h>

static char rcs_id[]= "$VER: $Id: ilbm2ascii.c,v 1.4 1994/06/15 20:43:46 tf Exp $";

void buythefarm(UBYTE *why, int how);
int point(struct BitMap *bm, short x, short y);
void PrBitMapASCII(FILE *fp, struct BitMap *bm);

/* System */

struct Library *GfxBase        = NULL;
struct Library *IFFParseBase   = NULL;

/* ILBM frame */
struct ILBMInfo ilbm = {0};

/* ILBM Property chunks to be grabbed - only BMHD needed for this app */
LONG ilbmprops[] = { ID_ILBM, ID_BMHD, TAG_DONE };

/* ILBM Collection chunks (more than one in file) to be gathered */
LONG *ilbmcollects = NULL;	/* none needed for this app */

/* ILBM Chunk to stop on */
LONG ilbmstops[] = { ID_ILBM, ID_BODY, TAG_DONE };


void main(int argc, char **argv)
{
  char *fname;
  long err;

  if(argc != 2 || argv[argc-1][0]=='?')
  {
    fprintf(stderr,"usage: %s ilbm-file  > ascii-file\n", argv[0]);
    exit(0);
  }

  fname= argv[1];

  /* Open Libraries */

  if(!(GfxBase = OpenLibrary("graphics.library",0)))
    buythefarm("Can't open graphics library",RETURN_WARN);

  if(!(IFFParseBase = OpenLibrary("iffparse.library",0)))
    buythefarm("Can't open iffparse library",RETURN_WARN);

  ilbm.ParseInfo.propchks	= ilbmprops;
  ilbm.ParseInfo.collectchks	= ilbmcollects;
  ilbm.ParseInfo.stopchks	= ilbmstops;

  /* Alloc IFF handle for frame */

  if(!(ilbm.ParseInfo.iff = AllocIFF()))
    buythefarm("Not enough memory!", RETURN_FAIL );
    /*buythefarm( IFFErr(IFFERR_NOMEM), RETURN_FAIL );*/

  if( err= loadbrush(&ilbm,fname) )
  {
    fprintf(stderr,"Can't load ILBM `%s'\n",fname,IFFerr(err));
    buythefarm("stop.",RETURN_WARN);
  }
  else /* Successfully loaded ILBM */
  {
    PrBitMapASCII( stdout, ilbm.brbitmap );
    unloadbrush(&ilbm);
  }
  buythefarm("done.",RETURN_OK);
}


void buythefarm(UBYTE *why, int how)
{
  fprintf(stderr,"%s\n",why);

  if(ilbm.ParseInfo.iff)  FreeIFF(ilbm.ParseInfo.iff);
  if(IFFParseBase)        CloseLibrary(IFFParseBase);
  if(GfxBase)             CloseLibrary(GfxBase);

  exit(how);
}


int point(struct BitMap *bm, short x, short y)
{
  int plane, pen;

  for(plane= pen= 0; plane < bm->Depth; plane++)
  {
    UBYTE b= ((UBYTE *)(bm->Planes[plane]))[y * bm->BytesPerRow + x/8];

    if( b & (1<<(7-(x%8))) )
      pen += (1<<plane);
  }

  return pen;
}


int magic(struct BitMap *bm, short x, short y)
{
  int m= 0;

  if( point(bm, x+0,y+0) ) m += (1<<0);
  if( point(bm, x+1,y+0) ) m += (1<<1);
  if( point(bm, x+0,y+1) ) m += (1<<2);
  if( point(bm, x+1,y+1) ) m += (1<<3);
  if( point(bm, x+0,y+2) ) m += (1<<4);
  if( point(bm, x+1,y+2) ) m += (1<<5);

  return m;
}


void PrBitMapASCII(FILE *fp, struct BitMap *bm)
{
  /* The Conversion Table is inspired by Jorn Barger's alt.ascii-art FAQ
     which had the table by Rob Harley(robert@vlsi.cs.caltech.edu) in it. */

  static char ctab[64]= " `'~-!/f-\\!V=+Y*.|/7i[/Pv)/ZzDZA,\\!Tct(5i\\]YeN4M_L2XsbKKgGd8mWW@";
  short x,y;

  char *lbuf= malloc( 4 * bm->BytesPerRow + 2 );

  if(lbuf)
  {
    for(y=0; y<bm->Rows; y+=3)
    {
      char *s, *t;

      for(s= t= lbuf, x=0; x < 8*bm->BytesPerRow; x+=2)
      {
        int m= magic(bm, x,y);
        *s++ = ctab[m];
        if(m) t=s;
      }

      *t++ = '\n';
      *t++ = '\0';

      fputs(lbuf,fp);
    }
    free(lbuf);
  }

  else /* no mem -> hyper slow ;) */
  {
    for(y=0; y<bm->Rows; y+=3)
    {
      for(x=0; x < 8*bm->BytesPerRow; x+=2)
      {
        fputc(ctab[magic(bm,x,y)],fp);
      }
      fputc('\n',fp);
    }
  }
}
